package com.icontrols.kakao.common.utils;

import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Base64;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.icontrols.kakao.common.KAKAO_RESULT;
import com.icontrols.kakao.common.SERVERINFO;

public class CommonUtil {
	public static boolean isWindows() {
		String OS = System.getProperty("os.name").toLowerCase();

		if (OS.indexOf("win") >= 0) {
			return true;
		}
		return false;
	}
	
	public static boolean getDevServer() {
		File checkFile = new File(SERVERINFO.DEV_SERVER);

		if (checkFile.exists()) {
			SERVERINFO.SendEventURL = SERVERINFO.SendDevEventURL;
			SERVERINFO.SendEventPort = SERVERINFO.SendDevEventPort;
			return true;
		} else
			return false;
	}
	
	
	public static boolean getUpdateNoticeNewDanjiServer() {
		File checkFile = new File(SERVERINFO.UPDATE_NOTICE_KIND);

		if (checkFile.exists())
			return true;
		else
			return false;
	}
	
	/**
	 * Gas Only One은 왜 그런걸까요오
	 * 나는 무엇을 위해 존재할 까요??
	 * @return
	 */
	public static boolean isGasOnlyOne() {
		String checkFileName;
		
		checkFileName = CommonUtil.isWindows()?SERVERINFO.WIN_ONLY_ONE_GAS:SERVERINFO.ONLY_ONE_GAS;
		
		File checkFile = new File(checkFileName);
		
		return checkFile.exists();
	}
	
	public static Map<String, Object> jsonToMap(String jsonString){
		ObjectMapper mapper = new ObjectMapper();
		Map<String, Object> map = new HashMap<String, Object>();
		
		//Convert JSON string to map
		try {
			map = mapper.readValue(jsonString, new TypeReference<Map<String, Object>>(){});
		} catch (JsonParseException e) {
			e.printStackTrace();
		} catch (JsonMappingException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		return map;
	}
	
	public static String getStrTypeCurTime() {
		Date now = new Date();
		String currentTime = new SimpleDateFormat("yyyy-MM-dd_HH:mm:ss").format(now);
		
		return currentTime;
	}
	
	public static String getStrTypeCurTimeAfterSec(int seconds) {
		Date today = new Date();
		Calendar cal = Calendar.getInstance();
		cal.setTime(today);
		cal.add(Calendar.SECOND, + seconds);
		
		SimpleDateFormat sdformat = new SimpleDateFormat("yyyy-MM-dd_HH:mm:ss"); 
		String EndTime = sdformat.format(cal.getTime());  
		
		return EndTime;
	}
	// String 타입의 두 날짜를 비교하기
	// -1 면 startTime 이 크다.
	// 0 이면 두 시간은 같다
	// 1 이면 endTime이 더 크다.
	public static int compareStringTypeTime(String startTime, String endTime) {
		try {
			SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd_HH:mm:ss");
			Date start_time = dateFormat.parse(startTime);
			Date end_time = dateFormat.parse(endTime);
			
			if(start_time.getTime() < end_time.getTime()) return 1;
			
			if(start_time.getTime() > end_time.getTime()) return -1;
		}catch(Exception e) {
			return -2;
		}
		return 0;
	}
	
	/**
	 * HTTP Header의 authorization 필드 비교 하여 client id : secret이 적합한것인지 확인
	 * @param authorization
	 * @return
	 */
	public static int validateHeaderAuth(String authorization) {
		String splitStr[] = authorization.split(" ");
		
		// splitStr length 가 2가 아니면 유효하지 않은 authorization 값
		if(splitStr.length != 2) return KAKAO_RESULT.FAIL_INVALID_CLIENT;
		
		// splitStr이 Basic이 아니어도 에러
		if(!splitStr[0].equals("Basic")) return KAKAO_RESULT.FAIL_INVALID_CLIENT;
		
		//Base64 디코드
		String decodeStr = new String(Base64.getDecoder().decode(splitStr[1]));
		
		splitStr = decodeStr.split(":");
		// splitStr length 가 2가 아니면 유효하지 않은 authorization 값
		if(splitStr.length != 2) return KAKAO_RESULT.FAIL_INVALID_CLIENT;
		
		if(splitStr[0].equals(SERVERINFO.KAKAO_HEADER_CLIENT_ID) && splitStr[1].equals(SERVERINFO.KAKAO_HEADER_CLIENT_SECRET)) {
			return KAKAO_RESULT.SUCCESS;
		}
		
		return KAKAO_RESULT.FAIL_INVALID_CLIENT;
	}
	
	public static String getClientIP(HttpServletRequest request) {
		String ip = request.getHeader("X-FORWARDED-FOR");
		
		if(ip == null || ip.length() == 0) {
			ip = request.getHeader("Proxy-Client-IP");
		}
		
		if(ip == null || ip.length() == 0) {
			ip = request.getHeader("WL-Proxy-Client-IP"); //웹 로직
		}
		
		if(ip == null || ip.length() == 0) {
			ip = request.getRemoteAddr();
		}
		
		return ip;
	}
}
